@@ -10,12 +10,6 @@
 #include "log.h"
 #include "util.h"
 
-uint32_t get_current_time_msec(void) {
-	struct timespec now;
-	clock_gettime(CLOCK_MONOTONIC, &now);
-	return now.tv_sec * 1000 + now.tv_nsec / 1000000;
-}
-
 int wrap(int i, int max) {
 	return ((i % max) + max) % max;
 }
