@@ -20,6 +20,7 @@ struct sway_layer_surface {
 	struct wl_listener surface_commit;
 	struct wl_listener output_destroy;
 	struct wl_listener new_popup;
+	struct wl_listener new_subsurface;
 
 	struct wlr_box geo;
 	enum zwlr_layer_shell_v1_layer layer;
@@ -39,6 +40,16 @@ struct sway_layer_popup {
 	struct wl_listener new_popup;
 };
 
+struct sway_layer_subsurface {
+	struct wlr_subsurface *wlr_subsurface;
+	struct sway_layer_surface *layer_surface;
+
+	struct wl_listener map;
+	struct wl_listener unmap;
+	struct wl_listener destroy;
+	struct wl_listener commit;
+};
+
 struct sway_output;
 void arrange_layers(struct sway_output *output);
 
