@@ -9,7 +9,9 @@ struct swaynag_type {
 	uint32_t anchors;
 
 	// Colors
+	uint32_t button_text;
 	uint32_t button_background;
+	uint32_t details_background;
 	uint32_t background;
 	uint32_t text;
 	uint32_t border;
