@@ -14,7 +14,7 @@ static struct cmd_results *do_split(int layout) {
 	struct sway_workspace *ws = config->handler_context.workspace;
 	if (con) {
 		if (container_is_scratchpad_hidden_or_child(con) &&
-				con->fullscreen_mode != FULLSCREEN_GLOBAL) {
+				con->pending.fullscreen_mode != FULLSCREEN_GLOBAL) {
 			return cmd_results_new(CMD_FAILURE,
 					"Cannot split a hidden scratchpad container");
 		}
@@ -23,10 +23,6 @@ static struct cmd_results *do_split(int layout) {
 		workspace_split(ws, layout);
 	}
 
-	if (con && con->parent && con->parent->parent) {
-		container_flatten(con->parent->parent);
-	}
-
 	if (root->fullscreen_global) {
 		arrange_root();
 	} else {
