@@ -19,17 +19,17 @@
 #include "sway/tree/workspace.h"
 
 static const char *atom_map[ATOM_LAST] = {
-	"_NET_WM_WINDOW_TYPE_NORMAL",
-	"_NET_WM_WINDOW_TYPE_DIALOG",
-	"_NET_WM_WINDOW_TYPE_UTILITY",
-	"_NET_WM_WINDOW_TYPE_TOOLBAR",
-	"_NET_WM_WINDOW_TYPE_SPLASH",
-	"_NET_WM_WINDOW_TYPE_MENU",
-	"_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
-	"_NET_WM_WINDOW_TYPE_POPUP_MENU",
-	"_NET_WM_WINDOW_TYPE_TOOLTIP",
-	"_NET_WM_WINDOW_TYPE_NOTIFICATION",
-	"_NET_WM_STATE_MODAL",
+	[NET_WM_WINDOW_TYPE_NORMAL] = "_NET_WM_WINDOW_TYPE_NORMAL",
+	[NET_WM_WINDOW_TYPE_DIALOG] = "_NET_WM_WINDOW_TYPE_DIALOG",
+	[NET_WM_WINDOW_TYPE_UTILITY] = "_NET_WM_WINDOW_TYPE_UTILITY",
+	[NET_WM_WINDOW_TYPE_TOOLBAR] = "_NET_WM_WINDOW_TYPE_TOOLBAR",
+	[NET_WM_WINDOW_TYPE_SPLASH] = "_NET_WM_WINDOW_TYPE_SPLASH",
+	[NET_WM_WINDOW_TYPE_MENU] = "_NET_WM_WINDOW_TYPE_MENU",
+	[NET_WM_WINDOW_TYPE_DROPDOWN_MENU] = "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
+	[NET_WM_WINDOW_TYPE_POPUP_MENU] = "_NET_WM_WINDOW_TYPE_POPUP_MENU",
+	[NET_WM_WINDOW_TYPE_TOOLTIP] = "_NET_WM_WINDOW_TYPE_TOOLTIP",
+	[NET_WM_WINDOW_TYPE_NOTIFICATION] = "_NET_WM_WINDOW_TYPE_NOTIFICATION",
+	[NET_WM_STATE_MODAL] = "_NET_WM_STATE_MODAL",
 };
 
 static void unmanaged_handle_request_configure(struct wl_listener *listener,
@@ -47,6 +47,15 @@ static void unmanaged_handle_commit(struct wl_listener *listener, void *data) {
 		wl_container_of(listener, surface, commit);
 	struct wlr_xwayland_surface *xsurface = surface->wlr_xwayland_surface;
 
+	desktop_damage_surface(xsurface->surface, surface->lx, surface->ly,
+		false);
+}
+
+static void unmanaged_handle_set_geometry(struct wl_listener *listener, void *data) {
+	struct sway_xwayland_unmanaged *surface =
+		wl_container_of(listener, surface, set_geometry);
+	struct wlr_xwayland_surface *xsurface = surface->wlr_xwayland_surface;
+
 	if (xsurface->x != surface->lx || xsurface->y != surface->ly) {
 		// Surface has moved
 		desktop_damage_surface(xsurface->surface, surface->lx, surface->ly,
@@ -55,9 +64,6 @@ static void unmanaged_handle_commit(struct wl_listener *listener, void *data) {
 		surface->ly = xsurface->y;
 		desktop_damage_surface(xsurface->surface, surface->lx, surface->ly,
 			true);
-	} else {
-		desktop_damage_surface(xsurface->surface, xsurface->x, xsurface->y,
-			false);
 	}
 }
 
@@ -68,6 +74,9 @@ static void unmanaged_handle_map(struct wl_listener *listener, void *data) {
 
 	wl_list_insert(root->xwayland_unmanaged.prev, &surface->link);
 
+	wl_signal_add(&xsurface->events.set_geometry, &surface->set_geometry);
+	surface->set_geometry.notify = unmanaged_handle_set_geometry;
+
 	wl_signal_add(&xsurface->surface->events.commit, &surface->commit);
 	surface->commit.notify = unmanaged_handle_commit;
 
@@ -89,25 +98,17 @@ static void unmanaged_handle_unmap(struct wl_listener *listener, void *data) {
 	struct wlr_xwayland_surface *xsurface = surface->wlr_xwayland_surface;
 	desktop_damage_surface(xsurface->surface, xsurface->x, xsurface->y, true);
 	wl_list_remove(&surface->link);
+	wl_list_remove(&surface->set_geometry.link);
 	wl_list_remove(&surface->commit.link);
 
 	struct sway_seat *seat = input_manager_current_seat();
-	if (seat->wlr_seat->keyboard_state.focused_surface ==
-			xsurface->surface) {
-
-		// Try to find another unmanaged surface from the same process to pass
-		// focus to. This is necessary because some applications (e.g. Jetbrains
-		// IDEs) represent their multi-level menus as unmanaged surfaces, and
-		// when closing a submenu, the main menu should get input focus.
-		struct sway_xwayland_unmanaged *current;
-		wl_list_for_each(current, &root->xwayland_unmanaged, link) {
-			struct wlr_xwayland_surface *prev_xsurface =
-				current->wlr_xwayland_surface;
-			if (prev_xsurface->pid == xsurface->pid &&
-					wlr_xwayland_or_surface_wants_focus(prev_xsurface)) {
-				seat_set_focus_surface(seat, prev_xsurface->surface, false);
-				return;
-			}
+	if (seat->wlr_seat->keyboard_state.focused_surface == xsurface->surface) {
+		// This simply returns focus to the parent surface if there's one available.
+		// This seems to handle JetBrains issues.
+		if (xsurface->parent && xsurface->parent->surface
+				&& wlr_xwayland_or_surface_wants_focus(xsurface->parent)) {
+			seat_set_focus_surface(seat, xsurface->parent->surface, false);
+			return;
 		}
 
 		// Restore focus
@@ -123,12 +124,36 @@ static void unmanaged_handle_unmap(struct wl_listener *listener, void *data) {
 static void unmanaged_handle_destroy(struct wl_listener *listener, void *data) {
 	struct sway_xwayland_unmanaged *surface =
 		wl_container_of(listener, surface, destroy);
+	wl_list_remove(&surface->request_configure.link);
 	wl_list_remove(&surface->map.link);
 	wl_list_remove(&surface->unmap.link);
 	wl_list_remove(&surface->destroy.link);
+	wl_list_remove(&surface->override_redirect.link);
 	free(surface);
 }
 
+static void handle_map(struct wl_listener *listener, void *data);
+
+struct sway_xwayland_view *create_xwayland_view(struct wlr_xwayland_surface *xsurface);
+
+static void unmanaged_handle_override_redirect(struct wl_listener *listener, void *data) {
+	struct sway_xwayland_unmanaged *surface =
+		wl_container_of(listener, surface, override_redirect);
+	struct wlr_xwayland_surface *xsurface = surface->wlr_xwayland_surface;
+
+	bool mapped = xsurface->mapped;
+	if (mapped) {
+		unmanaged_handle_unmap(&surface->unmap, NULL);
+	}
+
+	unmanaged_handle_destroy(&surface->destroy, NULL);
+	xsurface->data = NULL;
+	struct sway_xwayland_view *xwayland_view = create_xwayland_view(xsurface);
+	if (mapped) {
+		handle_map(&xwayland_view->map, xsurface);
+	}
+}
+
 static struct sway_xwayland_unmanaged *create_unmanaged(
 		struct wlr_xwayland_surface *xsurface) {
 	struct sway_xwayland_unmanaged *surface =
@@ -149,11 +174,12 @@ static struct sway_xwayland_unmanaged *create_unmanaged(
 	surface->unmap.notify = unmanaged_handle_unmap;
 	wl_signal_add(&xsurface->events.destroy, &surface->destroy);
 	surface->destroy.notify = unmanaged_handle_destroy;
+	wl_signal_add(&xsurface->events.set_override_redirect, &surface->override_redirect);
+	surface->override_redirect.notify = unmanaged_handle_override_redirect;
 
 	return surface;
 }
 
-
 static struct sway_xwayland_view *xwayland_view_from_view(
 		struct sway_view *view) {
 	if (!sway_assert(view->type == SWAY_VIEW_XWAYLAND,
@@ -222,6 +248,11 @@ static void set_activated(struct sway_view *view, bool activated) {
 		return;
 	}
 	struct wlr_xwayland_surface *surface = view->wlr_xwayland_surface;
+
+	if (activated && surface->minimized) {
+		wlr_xwayland_surface_set_minimized(surface, false);
+	}
+
 	wlr_xwayland_surface_activate(surface, activated);
 }
 
@@ -364,30 +395,31 @@ static void handle_commit(struct wl_listener *listener, void *data) {
 	struct wlr_xwayland_surface *xsurface = view->wlr_xwayland_surface;
 	struct wlr_surface_state *state = &xsurface->surface->current;
 
-	if (view->container->node.instruction) {
-		get_geometry(view, &view->geometry);
-		transaction_notify_view_ready_by_size(view,
-				state->width, state->height);
-	} else {
-		struct wlr_box new_geo;
-		get_geometry(view, &new_geo);
+	struct wlr_box new_geo;
+	get_geometry(view, &new_geo);
+	bool new_size = new_geo.width != view->geometry.width ||
+			new_geo.height != view->geometry.height ||
+			new_geo.x != view->geometry.x ||
+			new_geo.y != view->geometry.y;
 
-		if ((new_geo.width != view->geometry.width ||
-					new_geo.height != view->geometry.height ||
-					new_geo.x != view->geometry.x ||
-					new_geo.y != view->geometry.y)) {
-			// The view has unexpectedly sent a new size
-			// eg. The Firefox "Save As" dialog when downloading a file
-			desktop_damage_view(view);
-			view_update_size(view, new_geo.width, new_geo.height);
-			memcpy(&view->geometry, &new_geo, sizeof(struct wlr_box));
-			desktop_damage_view(view);
-			transaction_commit_dirty();
-			transaction_notify_view_ready_by_size(view,
-					new_geo.width, new_geo.height);
+	if (new_size) {
+		// The client changed its surface size in this commit. For floating
+		// containers, we resize the container to match. For tiling containers,
+		// we only recenter the surface.
+		desktop_damage_view(view);
+		memcpy(&view->geometry, &new_geo, sizeof(struct wlr_box));
+		if (container_is_floating(view->container)) {
+			view_update_size(view);
+			transaction_commit_dirty_client();
 		} else {
-			memcpy(&view->geometry, &new_geo, sizeof(struct wlr_box));
+			view_center_surface(view);
 		}
+		desktop_damage_view(view);
+	}
+
+	if (view->container->node.instruction) {
+		transaction_notify_view_ready_by_geometry(view,
+				xsurface->x, xsurface->y, state->width, state->height);
 	}
 
 	view_damage_from(view);
@@ -406,6 +438,7 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 	wl_list_remove(&xwayland_view->destroy.link);
 	wl_list_remove(&xwayland_view->request_configure.link);
 	wl_list_remove(&xwayland_view->request_fullscreen.link);
+	wl_list_remove(&xwayland_view->request_minimize.link);
 	wl_list_remove(&xwayland_view->request_move.link);
 	wl_list_remove(&xwayland_view->request_resize.link);
 	wl_list_remove(&xwayland_view->request_activate.link);
@@ -417,6 +450,7 @@ static void handle_destroy(struct wl_listener *listener, void *data) {
 	wl_list_remove(&xwayland_view->set_decorations.link);
 	wl_list_remove(&xwayland_view->map.link);
 	wl_list_remove(&xwayland_view->unmap.link);
+	wl_list_remove(&xwayland_view->override_redirect.link);
 	view_begin_destroy(&xwayland_view->view);
 }
 
@@ -440,16 +474,6 @@ static void handle_map(struct wl_listener *listener, void *data) {
 	struct wlr_xwayland_surface *xsurface = data;
 	struct sway_view *view = &xwayland_view->view;
 
-	if (xsurface->override_redirect) {
-		// This window used not to have the override redirect flag and has it
-		// now. Switch to unmanaged.
-		handle_destroy(&xwayland_view->destroy, view);
-		xsurface->data = NULL;
-		struct sway_xwayland_unmanaged *unmanaged = create_unmanaged(xsurface);
-		unmanaged_handle_map(&unmanaged->map, xsurface);
-		return;
-	}
-
 	view->natural_width = xsurface->width;
 	view->natural_height = xsurface->height;
 
@@ -464,6 +488,25 @@ static void handle_map(struct wl_listener *listener, void *data) {
 	transaction_commit_dirty();
 }
 
+static void handle_override_redirect(struct wl_listener *listener, void *data) {
+	struct sway_xwayland_view *xwayland_view =
+		wl_container_of(listener, xwayland_view, override_redirect);
+	struct wlr_xwayland_surface *xsurface = data;
+	struct sway_view *view = &xwayland_view->view;
+
+	bool mapped = xsurface->mapped;
+	if (mapped) {
+		handle_unmap(&xwayland_view->unmap, NULL);
+	}
+
+	handle_destroy(&xwayland_view->destroy, view);
+	xsurface->data = NULL;
+	struct sway_xwayland_unmanaged *unmanaged = create_unmanaged(xsurface);
+	if (mapped) {
+		unmanaged_handle_map(&unmanaged->map, xsurface);
+	}
+}
+
 static void handle_request_configure(struct wl_listener *listener, void *data) {
 	struct sway_xwayland_view *xwayland_view =
 		wl_container_of(listener, xwayland_view, request_configure);
@@ -481,10 +524,10 @@ static void handle_request_configure(struct wl_listener *listener, void *data) {
 		view->natural_height = ev->height;
 		container_floating_resize_and_center(view->container);
 
-		configure(view, view->container->content_x,
-				view->container->content_y,
-				view->container->content_width,
-				view->container->content_height);
+		configure(view, view->container->pending.content_x,
+				view->container->pending.content_y,
+				view->container->pending.content_width,
+				view->container->pending.content_height);
 		node_set_dirty(&view->container->node);
 	} else {
 		configure(view, view->container->current.content_x,
@@ -508,6 +551,21 @@ static void handle_request_fullscreen(struct wl_listener *listener, void *data)
 	transaction_commit_dirty();
 }
 
+static void handle_request_minimize(struct wl_listener *listener, void *data) {
+	struct sway_xwayland_view *xwayland_view =
+		wl_container_of(listener, xwayland_view, request_minimize);
+	struct sway_view *view = &xwayland_view->view;
+	struct wlr_xwayland_surface *xsurface = view->wlr_xwayland_surface;
+	if (!xsurface->mapped) {
+		return;
+	}
+
+	struct wlr_xwayland_minimize_event *e = data;
+	struct sway_seat *seat = input_manager_current_seat();
+	bool focused = seat_get_focus(seat) == &view->container->node;
+	wlr_xwayland_surface_set_minimized(xsurface, !focused && e->minimize);
+}
+
 static void handle_request_move(struct wl_listener *listener, void *data) {
 	struct sway_xwayland_view *xwayland_view =
 		wl_container_of(listener, xwayland_view, request_move);
@@ -621,22 +679,14 @@ struct sway_view *view_from_wlr_xwayland_surface(
 	return xsurface->data;
 }
 
-void handle_xwayland_surface(struct wl_listener *listener, void *data) {
-	struct wlr_xwayland_surface *xsurface = data;
-
-	if (xsurface->override_redirect) {
-		sway_log(SWAY_DEBUG, "New xwayland unmanaged surface");
-		create_unmanaged(xsurface);
-		return;
-	}
-
+struct sway_xwayland_view *create_xwayland_view(struct wlr_xwayland_surface *xsurface) {
 	sway_log(SWAY_DEBUG, "New xwayland surface title='%s' class='%s'",
 		xsurface->title, xsurface->class);
 
 	struct sway_xwayland_view *xwayland_view =
 		calloc(1, sizeof(struct sway_xwayland_view));
 	if (!sway_assert(xwayland_view, "Failed to allocate view")) {
-		return;
+		return NULL;
 	}
 
 	view_init(&xwayland_view->view, SWAY_VIEW_XWAYLAND, &view_impl);
@@ -653,6 +703,10 @@ void handle_xwayland_surface(struct wl_listener *listener, void *data) {
 		&xwayland_view->request_fullscreen);
 	xwayland_view->request_fullscreen.notify = handle_request_fullscreen;
 
+	wl_signal_add(&xsurface->events.request_minimize,
+		&xwayland_view->request_minimize);
+	xwayland_view->request_minimize.notify = handle_request_minimize;
+
 	wl_signal_add(&xsurface->events.request_activate,
 		&xwayland_view->request_activate);
 	xwayland_view->request_activate.notify = handle_request_activate;
@@ -691,7 +745,25 @@ void handle_xwayland_surface(struct wl_listener *listener, void *data) {
 	wl_signal_add(&xsurface->events.map, &xwayland_view->map);
 	xwayland_view->map.notify = handle_map;
 
+	wl_signal_add(&xsurface->events.set_override_redirect,
+			&xwayland_view->override_redirect);
+	xwayland_view->override_redirect.notify = handle_override_redirect;
+
 	xsurface->data = xwayland_view;
+
+	return xwayland_view;
+}
+
+void handle_xwayland_surface(struct wl_listener *listener, void *data) {
+	struct wlr_xwayland_surface *xsurface = data;
+
+	if (xsurface->override_redirect) {
+		sway_log(SWAY_DEBUG, "New xwayland unmanaged surface");
+		create_unmanaged(xsurface);
+		return;
+	}
+
+	create_xwayland_view(xsurface);
 }
 
 void handle_xwayland_ready(struct wl_listener *listener, void *data) {
