@@ -7,7 +7,7 @@
 #include <strings.h>
 #include <wayland-server-core.h>
 #include <wlr/types/wlr_output_layout.h>
-#include "cairo.h"
+#include "cairo_util.h"
 #include "pango.h"
 #include "sway/config.h"
 #include "sway/desktop.h"
@@ -531,6 +531,10 @@ static void update_title_texture(struct sway_container *con,
 	cairo_surface_destroy(dummy_surface);
 	cairo_destroy(c);
 
+	if (width == 0 || height == 0) {
+		return;
+	}
+
 	cairo_surface_t *surface = cairo_image_surface_create(
 			CAIRO_FORMAT_ARGB32, width, height);
 	cairo_t *cairo = cairo_create(surface);
@@ -1620,6 +1624,10 @@ static void update_marks_texture(struct sway_container *con,
 			"%s", buffer);
 	cairo_destroy(c);
 
+	if (width == 0 || height == 0) {
+		return;
+	}
+
 	cairo_surface_t *surface = cairo_image_surface_create(
 			CAIRO_FORMAT_ARGB32, width, height);
 	cairo_t *cairo = cairo_create(surface);
