@@ -56,6 +56,7 @@ void view_destroy(struct sway_view *view) {
 				"(might have a pending transaction?)")) {
 		return;
 	}
+	wl_list_remove(&view->events.unmap.listener_list);
 	if (!wl_list_empty(&view->saved_buffers)) {
 		view_remove_saved_buffer(view);
 	}
@@ -599,6 +600,11 @@ static bool should_focus(struct sway_view *view) {
 		return true;
 	}
 
+	// View opened "under" fullscreen view should not be given focus.
+	if (root->fullscreen_global || !map_ws || map_ws->fullscreen) {
+		return false;
+	}
+
 	// Views can only take focus if they are mapped into the active workspace
 	if (prev_ws != map_ws) {
 		return false;
@@ -638,6 +644,7 @@ static void handle_foreign_activate_request(
 			break;
 		}
 	}
+	transaction_commit_dirty();
 }
 
 static void handle_foreign_fullscreen_request(
@@ -677,6 +684,7 @@ static void handle_foreign_fullscreen_request(
 			arrange_workspace(container->pending.workspace);
 		}
 	}
+	transaction_commit_dirty();
 }
 
 static void handle_foreign_close_request(
@@ -757,7 +765,7 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 
 	view_init_subsurfaces(view, wlr_surface);
 	wl_signal_add(&wlr_surface->events.new_subsurface,
-		&view->surface_new_subsurface);
+			&view->surface_new_subsurface);
 	view->surface_new_subsurface.notify = view_handle_surface_new_subsurface;
 
 	if (decoration) {
@@ -805,9 +813,9 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 #if HAVE_XWAYLAND
 	if (wlr_surface_is_xwayland_surface(wlr_surface)) {
 		struct wlr_xwayland_surface *xsurface =
-			wlr_xwayland_surface_from_wlr_surface(wlr_surface);
-    	set_focus = (wlr_xwayland_icccm_input_model(xsurface) !=
-			WLR_ICCCM_INPUT_MODEL_NONE) && set_focus;
+				wlr_xwayland_surface_from_wlr_surface(wlr_surface);
+		set_focus &= wlr_xwayland_icccm_input_model(xsurface) !=
+				WLR_ICCCM_INPUT_MODEL_NONE;
 	}
 #endif
 
@@ -818,11 +826,9 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 	const char *app_id;
 	const char *class;
 	if ((app_id = view_get_app_id(view)) != NULL) {
-		wlr_foreign_toplevel_handle_v1_set_app_id(
-				view->foreign_toplevel, app_id);
+		wlr_foreign_toplevel_handle_v1_set_app_id(view->foreign_toplevel, app_id);
 	} else if ((class = view_get_class(view)) != NULL) {
-		wlr_foreign_toplevel_handle_v1_set_app_id(
-				view->foreign_toplevel, class);
+		wlr_foreign_toplevel_handle_v1_set_app_id(view->foreign_toplevel, class);
 	}
 }
 
@@ -1031,7 +1037,10 @@ static void view_child_handle_surface_destroy(struct wl_listener *listener,
 static void view_init_subsurfaces(struct sway_view *view,
 		struct wlr_surface *surface) {
 	struct wlr_subsurface *subsurface;
-	wl_list_for_each(subsurface, &surface->subsurfaces, parent_link) {
+	wl_list_for_each(subsurface, &surface->subsurfaces_below, parent_link) {
+		view_subsurface_create(view, subsurface);
+	}
+	wl_list_for_each(subsurface, &surface->subsurfaces_above, parent_link) {
 		view_subsurface_create(view, subsurface);
 	}
 }
@@ -1039,7 +1048,10 @@ static void view_init_subsurfaces(struct sway_view *view,
 static void view_child_init_subsurfaces(struct sway_view_child *view_child,
 		struct wlr_surface *surface) {
 	struct wlr_subsurface *subsurface;
-	wl_list_for_each(subsurface, &surface->subsurfaces, parent_link) {
+	wl_list_for_each(subsurface, &surface->subsurfaces_below, parent_link) {
+		view_child_subsurface_create(view_child, subsurface);
+	}
+	wl_list_for_each(subsurface, &surface->subsurfaces_above, parent_link) {
 		view_child_subsurface_create(view_child, subsurface);
 	}
 }
